<?php
class Lohnzettel {
	
	static function printLohnzettel(Mitarbeiter $ma, Lohnabrechnung $la, $path) {
		
		$name = $ma->getNachname();
		$fp = fopen($path, 'w');
		if ($fp === false) throw new Exception("Cannot open file: $path");
		$date = date("d.m.Y");
		$monat = self::convertMonat($la->getMonat());
		$jahr = substr($la->getMonat(), 0, 4);
		$vorname = $ma->getVorname();
		$strasse = $ma->getStrasse();
		$plz = $ma->getPlz();
		$ort = $ma->getOrt();
		$anrede = $ma->getAnrede();
		$geboren = $ma->getGebdat();
		$sozvernum = $ma->getSozVerNum();
		$steuerid = $ma->getSteuerid();
		$beschbeg = $ma->getBeschBeg();
		
		$text =<<<TEIL1










                                                               $date




Lohnabrechnung $monat
------------------------

$anrede
$vorname $name
$strasse
$plz $ort

Geboren am: $geboren
Sozialversicherungsnummer: $sozvernum
Steuer ID: $steuerid
Beschäftigungsbeginn: $beschbeg

TEIL1;
		$lp = $ma->getLohnprofil();
		if ($lp->getPeriode() == "Monat")
			$text = $text . sprintf("\n\n\n%-50s %10.2f\n", "Monatsgehalt", $la->getBrutto());
		else
			$text = $text . sprintf("\n\n\n%-50s %10.2f\n", "Arbeitszeit: ".$la->getArbeitsstunden()." h * ".$ma->getBasisBetrag(), $la->getBrutto());
		// TODO: Erst die Klasse abfragen, ob "getZulage" überhaupt definiert
		if (($betrag = $la->getZulage) > 0)
			$text = $text . sprintf("%-50s %10.2f\n", "Zulage:", $betrag);
		
		$text = $text . "\nAbzüge:\n";
		
		if (($betrag = $la->getLohnsteuer()) > 0)
			$text = $text . sprintf("%-50s %10.2f\n", "Lohnsteuer (".$ma->getSteuermerkmal()."):", $betrag);
			
		if (($betrag = $la->getKirchensteuerRk()) > 0)
			$text = $text . sprintf("%-50s %10.2f\n", "Kirchensteuer (rk):", $betrag);
				
		if (($betrag = $la->getKirchensteuerEv()) > 0)
			$text = $text . sprintf("%-50s %10.2f\n", "Kirchensteuer (ev):", $betrag);
			
		if (($betrag = $la->getSoli()) > 0)
			$text = $text . sprintf("%-51s %10.2f\n", "Solidaritätszuschlag:", $betrag); // ä sind 2 Byte
			
		if (($betrag = $la->getRentenversicherung()) > 0)
			$text = $text . sprintf("%-50s %10.2f\n", "Rentenversicherung:", $betrag);
			
		if (($betrag = $la->getKrankenversicherung()) > 0)
			$text = $text . sprintf("%-50s %10.2f\n", "Krankenversicherung:", $betrag);
			
		$netto = $la->getNetto();
		$konto = $ma->getIban();
		$bank = $ma->getBank();
		
		// bilde Jahressummen
		$bruttoSumme = $la->getJahressumme("brutto");
		
		$text = $text . "----------------------------------------------------------------\n";
		$text = $text . sprintf("%61.2f\n", $netto);
		
		$text = $text .<<<TEIL2
		                                           

Auszahlung erfolgt auf
Konto $konto
$bank




Summen $jahr (incl. akt. Monat):

TEIL2;
		$text = $text . sprintf("%-25s %10.2f\n", "Bruttogehalt:", $bruttoSumme);
		$text = $text . "\n";
		if (($betrag = $la->getJahressumme("lohnsteuer")) > 0)
			$text = $text . sprintf("%-25s %10.2f\n", "Lohnsteuer:", $betrag);
			
		if (($betrag = $la->getJahressumme("Kirchensteuer_Rk")) > 0)
			$text = $text . sprintf("%-25s %10.2f\n", "Kirchensteuer (rk):", $betrag);
		
		if (($betrag = $la->getJahressumme("Kirchensteuer_Ev")) > 0)
			$text = $text . sprintf("%-25s %10.2f\n", "Kirchensteuer (ev):", $betrag);
			
		if (($betrag = $la->getJahressumme("Soli")) > 0)
			$text = $text . sprintf("%-26s %10.2f\n", "Solidaritätszuschlag:", $betrag);
			
		if (($betrag = $la->getJahressumme("Rentenversicherung")) > 0)
			$text = $text . sprintf("%-25s %10.2f\n", "Rentenversicherung:", $betrag);
			
		if (($betrag = $la->getJahressumme("Krankenversicherung")) > 0)
			$text = $text . sprintf("%-25s %10.2f\n", "Krankenversicherung:", $betrag);
		
		$rc = fwrite($fp, $text);
		fclose($fp); 
		
		// check success
		if (!$rc) throw new Exception ("Cannot write to file: ".$path);
	}
	
	private static function convertMonat($monat) {
		$jahr = substr($monat, 0, 4);
		$mint = substr($monat, 4);
		switch ($mint) {
			case "01": return "Januar $jahr";
			case "02": return "Februar $jahr";
			case "03": return "März $jahr";
			case "04": return "April $jahr";
			case "05": return "Mai $jahr";
			case "06": return "Juni $jahr";
			case "07": return "Juli $jahr";
			case "08": return "August $jahr";
			case "09": return "September $jahr";
			case "10": return "Oktober $jahr";
			case "11": return "November $jahr";
			case "12": return "Dezember $jahr";
		}
	}
}
